@@ -1,6 +1,6 @@
 /*
  * xusb: Generic USB test program
- * Copyright  2009-2012 Pete Batard <pete@akeo.ie>
+ * Copyright © 2009-2012 Pete Batard <pete@akeo.ie>
  * Contributions to Mass Storage by Alan Stern.
  *
  * This library is free software; you can redistribute it and/or
@@ -80,9 +80,14 @@ static int perr(char const *format, ...)
 
 // HID Class-Specific Requests values. See section 7.2 of the HID specifications
 #define HID_GET_REPORT                0x01
+#define HID_GET_IDLE                  0x02
+#define HID_GET_PROTOCOL              0x03
 #define HID_SET_REPORT                0x09
+#define HID_SET_IDLE                  0x0A
+#define HID_SET_PROTOCOL              0x0B
 #define HID_REPORT_TYPE_INPUT         0x01
 #define HID_REPORT_TYPE_OUTPUT        0x02
+#define HID_REPORT_TYPE_FEATURE       0x03
 
 // Mass Storage Requests values. See section 3 of the Bulk-Only Mass Storage Class specifications
 #define BOMS_RESET                    0xFF
@@ -132,6 +137,7 @@ enum test_type {
 	USE_PS3,
 	USE_XBOX,
 	USE_SCSI,
+	USE_HID,
 } test_mode;
 uint16_t VID, PID;
 
@@ -525,6 +531,157 @@ static int test_mass_storage(libusb_device_handle *handle, uint8_t endpoint_in,
 	return 0;
 }
 
+// HID
+static int get_hid_record_size(uint8_t *hid_report_descriptor, int size, int type)
+{
+	uint8_t i, j = 0;
+	uint8_t offset;
+	int record_size[3] = {0, 0, 0};
+	int nb_bits = 0, nb_items = 0;
+	bool found_record_marker;
+
+	found_record_marker = false;
+	for (i = hid_report_descriptor[0]+1; i < size; i += offset) {
+		offset = (hid_report_descriptor[i]&0x03) + 1;
+		if (offset == 4)
+			offset = 5;
+		switch (hid_report_descriptor[i] & 0xFC) {
+		case 0x74:	// bitsize
+			nb_bits = hid_report_descriptor[i+1];
+			break;
+		case 0x94:	// count
+			nb_items = 0;
+			for (j=1; j<offset; j++) {
+				nb_items = ((uint32_t)hid_report_descriptor[i+j]) << (8*(j-1));
+			}
+			break;
+		case 0x80:	// input
+			found_record_marker = true;
+			j = 0;
+			break;
+		case 0x90:	// output
+			found_record_marker = true;
+			j = 1;
+			break;
+		case 0xb0:	// feature
+			found_record_marker = true;
+			j = 2;
+			break;
+		case 0xC0:	// end of collection
+			nb_items = 0;
+			nb_bits = 0;
+			break;
+		default:
+			continue;
+		}
+		if (found_record_marker) {
+			found_record_marker = false;
+			record_size[j] += nb_items*nb_bits;
+		}
+	}
+	if ((type < HID_REPORT_TYPE_INPUT) || (type > HID_REPORT_TYPE_FEATURE)) {
+		return 0;
+	} else {
+		return (record_size[type - HID_REPORT_TYPE_INPUT]+7)/8;
+	}
+}
+
+static int test_hid(libusb_device_handle *handle, uint8_t endpoint_in)
+{
+	int r, size, descriptor_size;
+	uint8_t hid_report_descriptor[256];
+	uint8_t *report_buffer;
+	FILE *fd;
+
+	printf("\nReading HID Report Descriptors:\n");
+	descriptor_size = libusb_control_transfer(handle, LIBUSB_ENDPOINT_IN|LIBUSB_REQUEST_TYPE_STANDARD|LIBUSB_RECIPIENT_INTERFACE,
+		LIBUSB_REQUEST_GET_DESCRIPTOR, LIBUSB_DT_REPORT<<8, 0, hid_report_descriptor, sizeof(hid_report_descriptor), 1000);
+	if (descriptor_size < 0) {
+		printf("   Failed\n");
+		return -1;
+	}
+	display_buffer_hex(hid_report_descriptor, descriptor_size);
+	if ((binary_dump) && ((fd = fopen(binary_name, "w")) != NULL)) {
+		if (fwrite(hid_report_descriptor, 1, descriptor_size, fd) != descriptor_size) {
+			printf("   Error writing descriptor to file\n");
+		}
+		fclose(fd);
+	}
+
+	size = get_hid_record_size(hid_report_descriptor, descriptor_size, HID_REPORT_TYPE_FEATURE);
+	if (size <= 0) {
+		printf("\nSkipping Feature Report readout (None detected)\n");
+	} else {
+		report_buffer = (uint8_t*) calloc(size, 1);
+		if (report_buffer == NULL) {
+			return -1;
+		}
+
+		printf("\nReading Feature Report (length %d)...\n", size);
+		r = libusb_control_transfer(handle, LIBUSB_ENDPOINT_IN|LIBUSB_REQUEST_TYPE_CLASS|LIBUSB_RECIPIENT_INTERFACE,
+			HID_GET_REPORT, (HID_REPORT_TYPE_FEATURE<<8)|0, 0, report_buffer, (uint16_t)size, 5000);
+		if (r >= 0) {
+			display_buffer_hex(report_buffer, size);
+		} else {
+			switch(r) {
+			case LIBUSB_ERROR_NOT_FOUND:
+				printf("   No Feature Report available for this device\n");
+				break;
+			case LIBUSB_ERROR_PIPE:
+				printf("   Detected stall - resetting pipe...\n");
+				libusb_clear_halt(handle, 0);
+				break;
+			default:
+				printf("   Error: %s\n", libusb_error_name(r));
+				break;
+			}
+		}
+		free(report_buffer);
+	}
+
+	size = get_hid_record_size(hid_report_descriptor, descriptor_size, HID_REPORT_TYPE_INPUT);
+	if (size <= 0) {
+		printf("\nSkipping Input Report readout (None detected)\n");
+	} else {
+		report_buffer = (uint8_t*) calloc(size, 1);
+		if (report_buffer == NULL) {
+			return -1;
+		}
+
+		printf("\nReading Input Report (length %d)...\n", size);
+		r = libusb_control_transfer(handle, LIBUSB_ENDPOINT_IN|LIBUSB_REQUEST_TYPE_CLASS|LIBUSB_RECIPIENT_INTERFACE,
+			HID_GET_REPORT, (HID_REPORT_TYPE_INPUT<<8)|0x00, 0, report_buffer, (uint16_t)size, 5000);
+		if (r >= 0) {
+			display_buffer_hex(report_buffer, size);
+		} else {
+			switch(r) {
+			case LIBUSB_ERROR_TIMEOUT:
+				printf("   Timeout! Please make sure you act on the device within the 5 seconds allocated...\n");
+				break;
+			case LIBUSB_ERROR_PIPE:
+				printf("   Detected stall - resetting pipe...\n");
+				libusb_clear_halt(handle, 0);
+				break;
+			default:
+				printf("   Error: %s\n", libusb_error_name(r));
+				break;
+			}
+		}
+
+		// Attempt a bulk read from endpoint 0 (this should just return a raw input report)
+		printf("\nTesting interrupt read using endpoint %02X...\n", endpoint_in);
+		r = libusb_interrupt_transfer(handle, endpoint_in, report_buffer, size, &size, 5000);
+		if (r >= 0) {
+			display_buffer_hex(report_buffer, size);
+		} else {
+			printf("   %s\n", libusb_error_name(r));
+		}
+
+		free(report_buffer);
+	}
+	return 0;
+}
+
 // Read the MS WinUSB Feature Descriptors, that are used on Windows 8 for automated driver installation
 static void read_ms_winsub_feature_descriptors(libusb_device_handle *handle, uint8_t bRequest, int iface_number)
 {
@@ -576,20 +733,18 @@ static int test_device(uint16_t vid, uint16_t pid)
 {
 	libusb_device_handle *handle;
 	libusb_device *dev;
-#ifdef HAS_GETPORTPATH
 	uint8_t bus, port_path[8];
-#endif
 	struct libusb_config_descriptor *conf_desc;
 	const struct libusb_endpoint_descriptor *endpoint;
 	int i, j, k, r;
 	int iface, nb_ifaces, first_iface = -1;
-#if defined(__linux)
+#if defined(__linux__)
 	// Attaching/detaching the kernel driver is only relevant for Linux
 	int iface_detached = -1;
 #endif
 	struct libusb_device_descriptor dev_desc;
-	const char* speed_name[5] = { "Unknown", "1.5 Mbit/s (USB 1.0 LowSpeed)", "12 Mbit/s (USB 1.0 FullSpeed)",
-		"480 Mbit/s (USB 2.0 HighSpeed)", "5000 Mbit/s (USB 3.0 SuperSpeed)"};
+	const char* speed_name[5] = { "Unknown", "1.5 Mbit/s (USB LowSpeed)", "12 Mbit/s (USB FullSpeed)",
+		"480 Mbit/s (USB HighSpeed)", "5000 Mbit/s (USB SuperSpeed)"};
 	char string[128];
 	uint8_t string_index[3];	// indexes of the string descriptors
 	uint8_t endpoint_in = 0, endpoint_out = 0;	// default IN and OUT endpoints
@@ -603,7 +758,6 @@ static int test_device(uint16_t vid, uint16_t pid)
 	}
 
 	dev = libusb_get_device(handle);
-#ifdef HAS_GETPORTPATH
 	bus = libusb_get_bus_number(dev);
 	r = libusb_get_port_path(NULL, dev, port_path, sizeof(port_path));
 	if (r > 0) {
@@ -613,7 +767,6 @@ static int test_device(uint16_t vid, uint16_t pid)
 		}
 		printf("\n");
 	}
-#endif
 	r = libusb_get_device_speed(dev);
 	if ((r<0) || (r>4)) r=0;
 	printf("speed: %s\n", speed_name[r]);
@@ -658,8 +811,8 @@ static int test_device(uint16_t vid, uint16_t pid)
 			for (k=0; k<conf_desc->usb_interface[i].altsetting[j].bNumEndpoints; k++) {
 				endpoint = &conf_desc->usb_interface[i].altsetting[j].endpoint[k];
 				printf("       endpoint[%d].address: %02X\n", k, endpoint->bEndpointAddress);
-				// Use the first bulk IN/OUT endpoints found as default for testing
-				if ((endpoint->bmAttributes & LIBUSB_TRANSFER_TYPE_MASK) == LIBUSB_TRANSFER_TYPE_BULK) {
+				// Use the first interrupt or bulk IN/OUT endpoints as default for testing
+				if ((endpoint->bmAttributes & LIBUSB_TRANSFER_TYPE_MASK) & (LIBUSB_TRANSFER_TYPE_BULK | LIBUSB_TRANSFER_TYPE_INTERRUPT)) {
 					if (endpoint->bEndpointAddress & LIBUSB_ENDPOINT_IN) {
 						if (!endpoint_in)
 							endpoint_in = endpoint->bEndpointAddress;
@@ -679,7 +832,7 @@ static int test_device(uint16_t vid, uint16_t pid)
 	{
 		printf("\nClaiming interface %d...\n", iface);
 		r = libusb_claim_interface(handle, iface);
-#if defined(__linux)
+#if defined(__linux__)
 		if ((r != LIBUSB_SUCCESS) && (iface == 0)) {
 			// Maybe we need to detach the driver
 			perr("   Failed. Trying to detach driver...\n");
@@ -722,6 +875,9 @@ static int test_device(uint16_t vid, uint16_t pid)
 		msleep(2000);
 		CALL_CHECK(set_xbox_actuators(handle, 0, 0));
 		break;
+	case USE_HID:
+		test_hid(handle, endpoint_in);
+		break;
 	case USE_SCSI:
 		CALL_CHECK(test_mass_storage(handle, endpoint_in, endpoint_out));
 	case USE_GENERIC:
@@ -734,7 +890,7 @@ static int test_device(uint16_t vid, uint16_t pid)
 		libusb_release_interface(handle, iface);
 	}
 
-#if defined(__linux)
+#if defined(__linux__)
 	if (iface_detached >= 0) {
 		printf("Re-attaching kernel driver...\n");
 		libusb_attach_kernel_driver(handle, iface_detached);
@@ -807,6 +963,12 @@ int main(int argc, char** argv)
 					PID = 0x0268;
 					test_mode = USE_PS3;
 					break;
+				case 's':
+					// Microsoft Sidewinder Precision Pro Joystick - 1 HID interface
+					VID = 0x045E;
+					PID = 0x0008;
+					test_mode = USE_HID;
+					break;
 				case 'x':
 					// Microsoft XBox Controller Type S - 1 interface
 					VID = 0x045E;
@@ -845,6 +1007,7 @@ int main(int argc, char** argv)
 		printf("   -k: test generic Mass Storage USB device (using WinUSB)\n");
 		printf("   -j: test FTDI based JTAG device (using WinUSB)\n");
 		printf("   -p: test Sony PS3 SixAxis controller (using WinUSB)\n");
+		printf("   -s: test Microsoft Sidewinder Precision Pro (using HID)\n");
 		printf("   -x: test Microsoft XBox Controller Type S (using WinUSB)\n");
 		return 0;
 	}
@@ -855,8 +1018,7 @@ int main(int argc, char** argv)
 	if (r < 0)
 		return r;
 
-	// Info = 3, Debug = 4
-	libusb_set_debug(NULL, debug_mode?4:3);
+	libusb_set_debug(NULL, debug_mode?LOG_LEVEL_DEBUG:LOG_LEVEL_INFO);
 
 	test_device(VID, PID);
 
