@@ -51,6 +51,7 @@ const struct libusb_version libusb_version_internal =
 	  LIBUSB_RC, "unused - please use the nano" };
 static int default_context_refcnt = 0;
 static usbi_mutex_static_t default_context_lock = USBI_MUTEX_INITIALIZER;
+static struct timeval timestamp_origin = { 0, 0 };
 
 /**
  * \mainpage libusbx-1.0 API Reference
@@ -110,8 +111,8 @@ static usbi_mutex_static_t default_context_lock = USBI_MUTEX_INITIALIZER;
  * free to close stdout/stderr and those descriptors may be reused without
  * worry.
  *
- * The libusb_set_debug() function can be used to enable stdout/stderr logging
- * of certain messages. Under standard configuration, libusbx doesn't really
+ * The libusb_set_debug() function can be used to enable stderr logging of
+ * certain messages. Under standard configuration, libusbx doesn't really
  * log much at all, so you are advised to use this function to enable all
  * error/warning/informational messages. It will help you debug problems with
  * your software.
@@ -120,7 +121,7 @@ static usbi_mutex_static_t default_context_lock = USBI_MUTEX_INITIALIZER;
  * between messages being logged and success or failure return codes from
  * libusbx functions. There is no format to the messages, so you should not
  * try to capture or parse them. They are not and will not be localized.
- * These messages are not suitable for being passed to your application user;
+ * These messages are not intended to being passed to your application user;
  * instead, you should interpret the error codes returned from libusbx functions
  * and provide appropriate notification to the user. The messages are simply
  * there to aid you as a programmer, and if you're confused because you're
@@ -128,8 +129,8 @@ static usbi_mutex_static_t default_context_lock = USBI_MUTEX_INITIALIZER;
  * logging may give you a suitable explanation.
  *
  * The LIBUSB_DEBUG environment variable can be used to enable message logging
- * at run-time. This environment variable should be set to a number, which is
- * interpreted the same as the libusb_set_debug() parameter. When this
+ * at run-time. This environment variable should be set to a log level number,
+ * which is interpreted the same as the libusb_set_debug() parameter. When this
  * environment variable is set, the message logging verbosity level is fixed
  * and libusb_set_debug() effectively does nothing.
  *
@@ -471,7 +472,7 @@ struct discovered_devs *discovered_devs_append(
 	/* exceeded capacity, need to grow */
 	usbi_dbg("need to increase capacity");
 	capacity = discdevs->capacity + DISCOVERED_DEVICES_SIZE_STEP;
-	discdevs = realloc(discdevs,
+	discdevs = usbi_reallocf(discdevs,
 		sizeof(*discdevs) + (sizeof(void *) * capacity));
 	if (discdevs) {
 		discdevs->capacity = capacity;
@@ -608,7 +609,7 @@ ssize_t API_EXPORTED libusb_get_device_list(libusb_context *ctx,
 
 	/* convert discovered_devs into a list */
 	len = discdevs->len;
-	ret = malloc(sizeof(void *) * (len + 1));
+	ret = calloc(len + 1, sizeof(struct libusb_device *));
 	if (!ret) {
 		len = LIBUSB_ERROR_NO_MEM;
 		goto out;
@@ -659,6 +660,71 @@ uint8_t API_EXPORTED libusb_get_bus_number(libusb_device *dev)
 	return dev->bus_number;
 }
 
+/** \ingroup dev
+ * Get the number of the port that a device is connected to
+ * \param dev a device
+ * \returns the port number (0 if not available)
+ */
+uint8_t API_EXPORTED libusb_get_port_number(libusb_device *dev)
+{
+	return dev->port_number;
+}
+
+/** \ingroup dev
+ * Get the list of all port numbers from root for the specified device
+ * \param dev a device
+ * \param path the array that should contain the port numbers
+ * \param path_len the maximum length of the array. As per the USB 3.0
+ * specs, the current maximum limit for the depth is 7.
+ * \returns the number of elements filled
+ * \returns LIBUSB_ERROR_OVERFLOW if the array is too small
+ */
+int API_EXPORTED libusb_get_port_path(libusb_context *ctx, libusb_device *dev, uint8_t* path, uint8_t path_len)
+{
+	int i = path_len;
+	ssize_t r;
+	struct libusb_device **devs;
+
+	/* The device needs to be open, else the parents may have been destroyed */
+	r = libusb_get_device_list(ctx, &devs);
+	if (r < 0)
+		return (int)r;
+
+	while(dev) {
+		// HCDs can be listed as devices and would have port #0
+		// TODO: see how the other backends want to implement HCDs as parents
+		if (dev->port_number == 0)
+			break;
+		i--;
+		if (i < 0) {
+			libusb_free_device_list(devs, 1);
+			return LIBUSB_ERROR_OVERFLOW;
+		}
+		path[i] = dev->port_number;
+		dev = dev->parent_dev;
+	}
+	libusb_free_device_list(devs, 1);
+	memmove(path, &path[i], path_len-i);
+	return path_len-i;
+}
+
+/** \ingroup dev
+ * Get the the parent from the specified device [EXPERIMENTAL]
+ * \param dev a device
+ * \returns the device parent or NULL if not available
+ * You should issue a libusb_get_device_list() before calling this
+ * function and make sure that you only access the parent before issuing
+ * libusb_free_device_list(). The reason is that libusbx currently does
+ * not maintain a permanent list of device instances, and therefore can
+ * only guarantee that parents are fully instantiated within a 
+ * libusb_get_device_list() - libusb_free_device_list() block.
+ */
+DEFAULT_VISIBILITY
+libusb_device * LIBUSB_CALL libusb_get_parent(libusb_device *dev)
+{
+	return dev->parent_dev;
+}
+
 /** \ingroup dev
  * Get the address of the device on the bus it is connected to.
  * \param dev a device
@@ -1499,20 +1565,16 @@ int API_EXPORTED libusb_attach_kernel_driver(libusb_device_handle *dev,
 }
 
 /** \ingroup lib
- * Set message verbosity.
- *  - Level 0: no messages ever printed by the library (default)
- *  - Level 1: error messages are printed to stderr
- *  - Level 2: warning and error messages are printed to stderr
- *  - Level 3: informational messages are printed to stdout, warning and error
- *    messages are printed to stderr
+ * Set log message verbosity.
  *
- * The default level is 0, which means no messages are ever printed. If you
- * choose to increase the message verbosity level, ensure that your
- * application does not close the stdout/stderr file descriptors.
+ * The default level is \ref LOG_LEVEL_NONE, which means no messages are ever
+ * printed. If you choose to increase the message verbosity level, ensure
+ * that your application does not close the stdout/stderr file descriptors.
  *
- * You are advised to set level 3. libusbx is conservative with its message
- * logging and most of the time, will only log messages that explain error
- * conditions and other oddities. This will help you debug your software.
+ * You are advised to use level \ref LOG_LEVEL_WARNING. libusbx is conservative
+ * with its message logging and most of the time, will only log messages that
+ * explain error conditions and other oddities. This will help you debug
+ * your software.
  *
  * If the LIBUSB_DEBUG environment variable was set when libusbx was
  * initialized, this function does nothing: the message verbosity is fixed
@@ -1554,6 +1616,11 @@ int API_EXPORTED libusb_init(libusb_context **context)
 	int r = 0;
 
 	usbi_mutex_static_lock(&default_context_lock);
+
+	if (!timestamp_origin.tv_sec) {
+		usbi_gettimeofday(&timestamp_origin, NULL);
+	}
+
 	if (!context && usbi_default_context) {
 		usbi_dbg("reusing default context");
 		default_context_refcnt++;
@@ -1568,13 +1635,24 @@ int API_EXPORTED libusb_init(libusb_context **context)
 	}
 	memset(ctx, 0, sizeof(*ctx));
 
+#ifdef ENABLE_DEBUG_LOGGING
+	ctx->debug = LOG_LEVEL_DEBUG;
+#endif
+
 	if (dbg) {
 		ctx->debug = atoi(dbg);
 		if (ctx->debug)
 			ctx->debug_fixed = 1;
 	}
 
-	usbi_dbg("");
+	/* default context should be initialized before calling usbi_dbg */
+	if (!usbi_default_context) {
+		usbi_default_context = ctx;
+		usbi_dbg("created default context");
+	}
+
+	usbi_dbg("libusbx v%d.%d.%d.%d", libusb_version_internal.major, libusb_version_internal.minor,
+		libusb_version_internal.micro, libusb_version_internal.nano);
 
 	if (usbi_backend->init) {
 		r = usbi_backend->init(ctx);
@@ -1718,63 +1796,71 @@ int usbi_gettimeofday(struct timeval *tp, void *tzp)
 void usbi_log_v(struct libusb_context *ctx, enum usbi_log_level level,
 	const char *function, const char *format, va_list args)
 {
-	FILE *stream = stdout;
-	const char *prefix;
+	const char *prefix = "";
 	struct timeval now;
-	static struct timeval first = { 0, 0 };
+	int global_debug;
+	static int has_debug_header_been_displayed = 0;
 
-#ifndef ENABLE_DEBUG_LOGGING
+#ifdef ENABLE_DEBUG_LOGGING
+	global_debug = 1;
+#else
 	USBI_GET_CONTEXT(ctx);
+	if (ctx == NULL)
+		return;
+	global_debug = (ctx->debug == LOG_LEVEL_DEBUG);
 	if (!ctx->debug)
 		return;
-	if (level == LOG_LEVEL_WARNING && ctx->debug < 2)
+	if (level == LOG_LEVEL_WARNING && ctx->debug < LOG_LEVEL_WARNING)
 		return;
-	if (level == LOG_LEVEL_INFO && ctx->debug < 3)
+	if (level == LOG_LEVEL_INFO && ctx->debug < LOG_LEVEL_INFO)
+		return;
+	if (level == LOG_LEVEL_DEBUG && ctx->debug < LOG_LEVEL_DEBUG)
 		return;
 #endif
 
 	usbi_gettimeofday(&now, NULL);
-	if (!first.tv_sec) {
-		first.tv_sec = now.tv_sec;
-		first.tv_usec = now.tv_usec;
-		fprintf(stream, "[timestamp] [threadID] facility level [function call] <message>\n");
-		fprintf(stream, "--------------------------------------------------------------------------------\n");
+	if ((global_debug) && (!has_debug_header_been_displayed)) {
+		has_debug_header_been_displayed = 1;
+		fprintf(stderr, "[timestamp] [threadID] facility level [function call] <message>\n");
+		fprintf(stderr, "--------------------------------------------------------------------------------\n");
 	}
-	if (now.tv_usec < first.tv_usec) {
+	if (now.tv_usec < timestamp_origin.tv_usec) {
 		now.tv_sec--;
 		now.tv_usec += 1000000;
 	}
-	now.tv_sec -= first.tv_sec;
-	now.tv_usec -= first.tv_usec;
+	now.tv_sec -= timestamp_origin.tv_sec;
+	now.tv_usec -= timestamp_origin.tv_usec;
 
 	switch (level) {
 	case LOG_LEVEL_INFO:
 		prefix = "info";
 		break;
 	case LOG_LEVEL_WARNING:
-		stream = stderr;
 		prefix = "warning";
 		break;
 	case LOG_LEVEL_ERROR:
-		stream = stderr;
 		prefix = "error";
 		break;
 	case LOG_LEVEL_DEBUG:
-		stream = stderr;
 		prefix = "debug";
 		break;
+	case LOG_LEVEL_NONE:
+		break;
 	default:
-		stream = stderr;
 		prefix = "unknown";
 		break;
 	}
 
-	fprintf(stream, "[%2d.%06d] [%08x] libusbx: %s [%s] ",
-		(int)now.tv_sec, (int)now.tv_usec, usbi_get_tid(), prefix, function);
+	if (global_debug) {
+		fprintf(stderr, "[%2d.%06d] [%08x] libusbx: %s [%s] ",
+			(int)now.tv_sec, (int)now.tv_usec, usbi_get_tid(), prefix, function);
+	} else {
+		fprintf(stderr, "libusbx: %s [%s] ", prefix, function);
+	}
 
-	vfprintf(stream, format, args);
+	vfprintf(stderr, format, args);
 
-	fprintf(stream, "\n");
+	fprintf(stderr, "\n");
 }
 
 void usbi_log(struct libusb_context *ctx, enum usbi_log_level level,
