@@ -1,6 +1,6 @@
 /*
  * darwin backend for libusbx 1.0
- * Copyright  2008-2011 Nathan Hjelm <hjelmn@users.sourceforge.net>
+ * Copyright © 2008-2011 Nathan Hjelm <hjelmn@users.sourceforge.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -67,6 +67,7 @@ static int darwin_release_interface(struct libusb_device_handle *dev_handle, int
 static int darwin_reset_device(struct libusb_device_handle *dev_handle);
 static void darwin_async_io_callback (void *refcon, IOReturn result, void *arg0);
 
+#ifdef ENABLE_LOGGING
 static const char *darwin_error_str (int result) {
   switch (result) {
   case kIOReturnSuccess:
@@ -99,6 +100,7 @@ static const char *darwin_error_str (int result) {
     return "unknown error";
   }
 }
+#endif
 
 static int darwin_to_libusb (int result) {
   switch (result) {
@@ -134,7 +136,7 @@ static int ep_to_pipeRef(struct libusb_device_handle *dev_handle, uint8_t ep, ui
 
   int8_t i, iface;
 
-  usbi_info (HANDLE_CTX(dev_handle), "converting ep address 0x%02x to pipeRef and interface", ep);
+  usbi_dbg ("converting ep address 0x%02x to pipeRef and interface", ep);
 
   for (iface = 0 ; iface < USB_MAXINTERFACES ; iface++) {
     cInterface = &priv->interfaces[iface];
@@ -144,7 +146,7 @@ static int ep_to_pipeRef(struct libusb_device_handle *dev_handle, uint8_t ep, ui
 	if (cInterface->endpoint_addrs[i] == ep) {
 	  *pipep = i + 1;
 	  *ifcp = iface;
-	  usbi_info (HANDLE_CTX(dev_handle), "pipe %d on interface %d matches", *pipep, *ifcp);
+	  usbi_dbg ("pipe %d on interface %d matches", *pipep, *ifcp);
 	  return 0;
 	}
       }
@@ -185,11 +187,26 @@ static int usb_setup_device_iterator (io_iterator_t *deviceIterator, long locati
   return IOServiceGetMatchingServices(kIOMasterPortDefault, matchingDict, deviceIterator);
 }
 
-static usb_device_t **usb_get_next_device (io_iterator_t deviceIterator, UInt32 *locationp) {
+static int get_ioregistry_value_number (io_service_t service, CFStringRef property, CFNumberType type, void *p) {
+  CFTypeRef cfNumber = IORegistryEntryCreateCFProperty (service, property, kCFAllocatorDefault, 0);
+  int ret = 0;
+
+  if (cfNumber) {
+    if (CFGetTypeID(cfNumber) == CFNumberGetTypeID()) {
+      ret = CFNumberGetValue(cfNumber, type, p);
+    }
+
+    CFRelease (cfNumber);
+  }
+
+  return ret;
+}
+
+static usb_device_t **usb_get_next_device (io_iterator_t deviceIterator, UInt32 *locationp, UInt8 *portp, UInt32 *parent_locationp) {
   io_cf_plugin_ref_t *plugInInterface = NULL;
   usb_device_t **device;
-  io_service_t usbDevice;
-  long result;
+  io_service_t usbDevice, parent;
+  kern_return_t result;
   SInt32 score;
 
   if (!IOIteratorIsValid (deviceIterator))
@@ -203,6 +220,22 @@ static usb_device_t **usb_get_next_device (io_iterator_t deviceIterator, UInt32
 
     /* we are done with the usb_device_t */
     (void)IOObjectRelease(usbDevice);
+
+    if (portp) {
+      *portp = 0;
+      (void) get_ioregistry_value_number (usbDevice, CFSTR("PortNum"), kCFNumberSInt8Type, portp);
+    }
+
+    if (parent_locationp) {
+      *parent_locationp = 0;
+
+      result = IORegistryEntryGetParentEntry (usbDevice, kIOUSBPlane, &parent);
+
+      if (kIOReturnSuccess == result) {
+	(void) get_ioregistry_value_number (parent, CFSTR("locationID"), kCFNumberLongType, parent_locationp);
+      }
+    }
+
     if (kIOReturnSuccess == result && plugInInterface)
       break;
 
@@ -235,7 +268,7 @@ static kern_return_t darwin_get_device (uint32_t dev_location, usb_device_t ***d
     return kresult;
 
   /* This port of libusb uses locations to keep track of devices. */
-  while ((*darwin_device = usb_get_next_device (deviceIterator, &location)) != NULL) {
+  while ((*darwin_device = usb_get_next_device (deviceIterator, &location, NULL, NULL)) != NULL) {
     if (location == dev_location)
       break;
 
@@ -264,7 +297,7 @@ static void darwin_devices_detached (void *ptr, io_iterator_t rem_devices) {
   CFTypeRef locationCF;
   UInt32 message;
 
-  usbi_info (ctx, "a device has been detached");
+  usbi_dbg ("a device has been detached");
 
   while ((device = IOIteratorNext (rem_devices)) != 0) {
     /* get the location from the i/o registry */
@@ -331,7 +364,7 @@ static void *event_thread_main (void *arg0) {
   io_notification_port_t libusb_notification_port;
   io_iterator_t          libusb_rem_device_iterator;
 
-  usbi_info (ctx, "creating hotplug event source");
+  usbi_dbg ("creating hotplug event source");
 
   runloop = CFRunLoopGetCurrent ();
   CFRetain (runloop);
@@ -356,7 +389,7 @@ static void *event_thread_main (void *arg0) {
   /* arm notifiers */
   darwin_clear_iterator (libusb_rem_device_iterator);
 
-  usbi_info (ctx, "thread ready to receive events");
+  usbi_dbg ("thread ready to receive events");
 
   /* signal the main thread that the async runloop has been created. */
   pthread_mutex_lock (&libusb_darwin_at_mutex);
@@ -367,7 +400,7 @@ static void *event_thread_main (void *arg0) {
   /* run the runloop */
   CFRunLoopRun();
 
-  usbi_info (ctx, "thread exiting");
+  usbi_dbg ("thread exiting");
 
   /* delete notification port */
   IONotificationPortDestroy (libusb_notification_port);
@@ -555,7 +588,7 @@ static int darwin_check_configuration (struct libusb_context *ctx, struct libusb
     /* not configured */
     priv->active_config = 0;
   
-  usbi_info (ctx, "active config: %u, first config: %u", priv->active_config, priv->first_config);
+  usbi_dbg ("active config: %u, first config: %u", priv->active_config, priv->first_config);
 
   return 0;
 }
@@ -691,9 +724,11 @@ static int darwin_cache_device_descriptor (struct libusb_context *ctx, struct li
   return 0;
 }
 
-static int process_new_device (struct libusb_context *ctx, usb_device_t **device, UInt32 locationID, struct discovered_devs **_discdevs) {
+static int process_new_device (struct libusb_context *ctx, usb_device_t **device, UInt32 locationID,
+			       UInt32 parent_location, UInt8 port, struct discovered_devs **_discdevs,
+			       struct libusb_device **last_dev) {
   struct darwin_device_priv *priv;
-  struct libusb_device *dev;
+  struct libusb_device *dev, *parent = NULL;
   struct discovered_devs *discdevs;
   UInt16                address;
   UInt8                 devSpeed;
@@ -702,11 +737,11 @@ static int process_new_device (struct libusb_context *ctx, usb_device_t **device
   do {
     dev = usbi_get_device_by_session_id(ctx, locationID);
     if (!dev) {
-      usbi_info (ctx, "allocating new device for location 0x%08x", locationID);
+      usbi_dbg ("allocating new device for location 0x%08x", locationID);
       dev = usbi_alloc_device(ctx, locationID);
       need_unref = 1;
     } else
-      usbi_info (ctx, "using existing device for location 0x%08x", locationID);
+      usbi_dbg ("using existing device for location 0x%08x", locationID);
 
     if (!dev) {
       ret = LIBUSB_ERROR_NO_MEM;
@@ -726,6 +761,19 @@ static int process_new_device (struct libusb_context *ctx, usb_device_t **device
     if (ret < 0)
       break;
 
+    /* the device iterator provides devices in increasing order of location. given this property
+     * we can use the last device to find the parent. */
+    for (parent = *last_dev ; parent ; parent = parent->parent_dev) {
+      struct darwin_device_priv *parent_priv = (struct darwin_device_priv *) parent->os_priv;
+
+      if (parent_priv->location == parent_location) {
+	break;
+      }
+    }
+
+    dev->parent_dev = parent;
+
+    dev->port_number    = port;
     dev->bus_number     = locationID >> 24;
     dev->device_address = address;
 
@@ -756,8 +804,10 @@ static int process_new_device (struct libusb_context *ctx, usb_device_t **device
     }
 
     *_discdevs = discdevs;
+    *last_dev = dev;
 
-    usbi_info (ctx, "found device with address %d at %s", dev->device_address, priv->sys_path);
+    usbi_dbg ("found device with address %d port = %d parent = %p at %p", dev->device_address,
+	       dev->port_number, priv->sys_path, (void *) parent);
   } while (0);
 
   if (need_unref)
@@ -770,14 +820,16 @@ static int darwin_get_device_list(struct libusb_context *ctx, struct discovered_
   io_iterator_t        deviceIterator;
   usb_device_t         **device;
   kern_return_t        kresult;
-  UInt32               location;
+  UInt32               location, parent_location;
+  UInt8                port;
+  struct libusb_device *last_dev = NULL;
 
   kresult = usb_setup_device_iterator (&deviceIterator, 0);
   if (kresult != kIOReturnSuccess)
     return darwin_to_libusb (kresult);
 
-  while ((device = usb_get_next_device (deviceIterator, &location)) != NULL) {
-    (void) process_new_device (ctx, device, location, _discdevs);
+  while ((device = usb_get_next_device (deviceIterator, &location, &port, &parent_location)) != NULL) {
+    (void) process_new_device (ctx, device, location, parent_location, port, _discdevs, &last_dev);
 
     (*(device))->Release(device);
   }
@@ -806,7 +858,7 @@ static int darwin_open (struct libusb_device_handle *dev_handle) {
     kresult = (*(dpriv->device))->USBDeviceOpenSeize (dpriv->device);
 
     if (kresult != kIOReturnSuccess) {
-      usbi_err (HANDLE_CTX (dev_handle), "USBDeviceOpen: %s", darwin_error_str(kresult));
+      usbi_warn (HANDLE_CTX (dev_handle), "USBDeviceOpen: %s", darwin_error_str(kresult));
 
       switch (kresult) {
       case kIOReturnExclusiveAccess:
@@ -852,7 +904,7 @@ static int darwin_open (struct libusb_device_handle *dev_handle) {
 
   usbi_add_pollfd(HANDLE_CTX(dev_handle), priv->fds[0], POLLIN);
 
-  usbi_info (HANDLE_CTX (dev_handle), "device open for access");
+  usbi_dbg ("device open for access");
 
   return 0;
 }
@@ -889,7 +941,7 @@ static void darwin_close (struct libusb_device_handle *dev_handle) {
       if (kresult) {
 	/* Log the fact that we had a problem closing the file, however failing a
 	 * close isn't really an error, so return success anyway */
-	usbi_err (HANDLE_CTX (dev_handle), "USBDeviceClose: %s", darwin_error_str(kresult));
+	usbi_warn (HANDLE_CTX (dev_handle), "USBDeviceClose: %s", darwin_error_str(kresult));
       }
     }
 
@@ -897,7 +949,7 @@ static void darwin_close (struct libusb_device_handle *dev_handle) {
     if (kresult) {
       /* Log the fact that we had a problem closing the file, however failing a
        * close isn't really an error, so return success anyway */
-      usbi_err (HANDLE_CTX (dev_handle), "Release: %s", darwin_error_str(kresult));
+      usbi_warn (HANDLE_CTX (dev_handle), "Release: %s", darwin_error_str(kresult));
     }
 
     dpriv->device = NULL;
@@ -987,7 +1039,7 @@ static int get_endpoints (struct libusb_device_handle *dev_handle, int iface) {
   u_int16_t dont_care2;
   int i;
 
-  usbi_info (HANDLE_CTX (dev_handle), "building table of endpoints.");
+  usbi_dbg ("building table of endpoints.");
 
   /* retrieve the total number of endpoints on this interface */
   kresult = (*(cInterface->interface))->GetNumEndpoints(cInterface->interface, &numep);
@@ -1007,7 +1059,7 @@ static int get_endpoints (struct libusb_device_handle *dev_handle, int iface) {
       return darwin_to_libusb (kresult);
     }
 
-    usbi_info (HANDLE_CTX (dev_handle), "interface: %i pipe %i: dir: %i number: %i", iface, i, direction, number);
+    usbi_dbg ("interface: %i pipe %i: dir: %i number: %i", iface, i, direction, number);
 
     cInterface->endpoint_addrs[i - 1] = ((direction << 7 & LIBUSB_ENDPOINT_DIR_MASK) | (number & LIBUSB_ENDPOINT_ADDRESS_MASK));
   }
@@ -1115,7 +1167,7 @@ static int darwin_claim_interface(struct libusb_device_handle *dev_handle, int i
   /* add the cfSource to the async thread's run loop */
   CFRunLoopAddSource(libusb_darwin_acfl, cInterface->cfSource, kCFRunLoopDefaultMode);
 
-  usbi_info (HANDLE_CTX (dev_handle), "interface opened");
+  usbi_dbg ("interface opened");
 
   return 0;
 }
@@ -1142,11 +1194,11 @@ static int darwin_release_interface(struct libusb_device_handle *dev_handle, int
 
   kresult = (*(cInterface->interface))->USBInterfaceClose(cInterface->interface);
   if (kresult)
-    usbi_err (HANDLE_CTX (dev_handle), "USBInterfaceClose: %s", darwin_error_str(kresult));
+    usbi_warn (HANDLE_CTX (dev_handle), "USBInterfaceClose: %s", darwin_error_str(kresult));
 
   kresult = (*(cInterface->interface))->Release(cInterface->interface);
   if (kresult != kIOReturnSuccess)
-    usbi_err (HANDLE_CTX (dev_handle), "Release: %s", darwin_error_str(kresult));
+    usbi_warn (HANDLE_CTX (dev_handle), "Release: %s", darwin_error_str(kresult));
 
   cInterface->interface = IO_OBJECT_NULL;
 
@@ -1203,7 +1255,7 @@ static int darwin_clear_halt(struct libusb_device_handle *dev_handle, unsigned c
   kresult = (*(cInterface->interface))->ClearPipeStallBothEnds(cInterface->interface, pipeRef);
 #endif
   if (kresult)
-    usbi_err (HANDLE_CTX (dev_handle), "ClearPipeStall: %s", darwin_error_str (kresult));
+    usbi_warn (HANDLE_CTX (dev_handle), "ClearPipeStall: %s", darwin_error_str (kresult));
 
   return darwin_to_libusb (kresult);
 }
@@ -1466,7 +1518,7 @@ static int cancel_control_transfer(struct usbi_transfer *itransfer) {
   struct darwin_device_priv *dpriv = (struct darwin_device_priv *)transfer->dev_handle->dev->os_priv;
   IOReturn kresult;
 
-  usbi_info (ITRANSFER_CTX (itransfer), "WARNING: aborting all transactions control pipe");
+  usbi_warn (ITRANSFER_CTX (itransfer), "aborting all transactions control pipe");
 
   if (!dpriv->device)
     return LIBUSB_ERROR_NO_DEVICE;
@@ -1495,12 +1547,12 @@ static int darwin_abort_transfers (struct usbi_transfer *itransfer) {
   if (!dpriv->device)
     return LIBUSB_ERROR_NO_DEVICE;
 
-  usbi_info (ITRANSFER_CTX (itransfer), "WARNING: aborting all transactions on interface %d pipe %d", iface, pipeRef);
+  usbi_warn (ITRANSFER_CTX (itransfer), "aborting all transactions on interface %d pipe %d", iface, pipeRef);
 
   /* abort transactions */
   (*(cInterface->interface))->AbortPipe (cInterface->interface, pipeRef);
 
-  usbi_info (ITRANSFER_CTX (itransfer), "calling clear pipe stall to clear the data toggle bit");
+  usbi_dbg ("calling clear pipe stall to clear the data toggle bit");
 
   /* clear the data toggle bit */
 #if (InterfaceVersion < 190)
@@ -1545,7 +1597,7 @@ static void darwin_async_io_callback (void *refcon, IOReturn result, void *arg0)
   struct darwin_device_handle_priv *priv = (struct darwin_device_handle_priv *)transfer->dev_handle->os_priv;
   UInt32 message, size;
 
-  usbi_info (ITRANSFER_CTX (itransfer), "an async io operation has completed");
+  usbi_dbg ("an async io operation has completed");
 
   /* The size should never be larger than 4 GB - Also see libusb bug #117 */
   if ((intptr_t) arg0 > UINT32_MAX)
@@ -1572,17 +1624,17 @@ static int darwin_transfer_status (struct usbi_transfer *itransfer, kern_return_
   case kIOReturnAborted:
     return LIBUSB_TRANSFER_CANCELLED;
   case kIOUSBPipeStalled:
-    usbi_warn (ITRANSFER_CTX (itransfer), "transfer error: pipe is stalled");
+    usbi_dbg ("transfer error: pipe is stalled");
     return LIBUSB_TRANSFER_STALL;
   case kIOReturnOverrun:
-    usbi_err (ITRANSFER_CTX (itransfer), "transfer error: data overrun");
+    usbi_warn (ITRANSFER_CTX (itransfer), "transfer error: data overrun");
     return LIBUSB_TRANSFER_OVERFLOW;
   case kIOUSBTransactionTimeout:
-    usbi_err (ITRANSFER_CTX (itransfer), "transfer error: timed out");
+    usbi_warn (ITRANSFER_CTX (itransfer), "transfer error: timed out");
     itransfer->flags |= USBI_TRANSFER_TIMED_OUT;
     return LIBUSB_TRANSFER_TIMED_OUT;
   default:
-    usbi_err (ITRANSFER_CTX (itransfer), "transfer error: %s (value = 0x%08x)", darwin_error_str (result), result);
+    usbi_warn (ITRANSFER_CTX (itransfer), "transfer error: %s (value = 0x%08x)", darwin_error_str (result), result);
     return LIBUSB_TRANSFER_ERROR;
   }
 }
@@ -1601,7 +1653,7 @@ static void darwin_handle_callback (struct usbi_transfer *itransfer, kern_return
     return;
   }
 
-  usbi_info (ITRANSFER_CTX (itransfer), "handling %s completion with kernel status %d",
+  usbi_dbg ("handling %s completion with kernel status %d",
 	     isControl ? "control" : isBulk ? "bulk" : isIsoc ? "isoc" : "interrupt", result);
 
   if (kIOReturnSuccess == result || kIOReturnUnderrun == result) {
@@ -1635,7 +1687,7 @@ static int op_handle_events(struct libusb_context *ctx, struct pollfd *fds, POLL
     struct libusb_device_handle *handle;
     struct darwin_device_handle_priv *hpriv = NULL;
 
-    usbi_info (ctx, "checking fd %i with revents = %x", fds[i], pollfd->revents);
+    usbi_dbg ("checking fd %i with revents = %x", fds[i], pollfd->revents);
 
     if (!pollfd->revents)
       continue;
@@ -1678,7 +1730,7 @@ static int op_handle_events(struct libusb_context *ctx, struct pollfd *fds, POLL
       darwin_handle_callback (itransfer, kresult, io_size);
       break;
     default:
-      usbi_err (ctx, "unknown message received from device pipe");
+      usbi_warn (ctx, "unknown message received from device pipe");
     }
   }
 
